@@ -1,9 +1,5 @@
-from __future__ import unicode_literals
-
 import datetime
-import warnings
 
-from django.contrib.admin.templatetags.admin_static import static
 from django.contrib.admin.templatetags.admin_urls import add_preserved_filters
 from django.contrib.admin.utils import (
     display_for_field, display_for_value, label_for_field, lookup_field,
@@ -12,17 +8,16 @@ from django.contrib.admin.views.main import (
     ALL_VAR, ORDER_VAR, PAGE_VAR, SEARCH_VAR,
 )
 from django.core.exceptions import ObjectDoesNotExist
-from django.core.urlresolvers import NoReverseMatch
 from django.db import models
 from django.template import Library
 from django.template.loader import get_template
+from django.templatetags.static import static
+from django.urls import NoReverseMatch
 from django.utils import formats
-from django.utils.deprecation import RemovedInDjango20Warning
-from django.utils.encoding import force_text
-from django.utils.html import escapejs, format_html
+from django.utils.html import format_html
 from django.utils.safestring import mark_safe
 from django.utils.text import capfirst
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 
 register = Library()
 
@@ -32,7 +27,7 @@ DOT = '.'
 @register.simple_tag
 def paginator_number(cl, i):
     """
-    Generates an individual page index link in a paginated list.
+    Generate an individual page index link in a paginated list.
     """
     if i == DOT:
         return '... '
@@ -48,7 +43,7 @@ def paginator_number(cl, i):
 @register.inclusion_tag('admin/pagination.html')
 def pagination(cl):
     """
-    Generates the series of links to the pages in a paginated list.
+    Generate the series of links to the pages in a paginated list.
     """
     paginator, page_num = cl.paginator, cl.page_num
 
@@ -94,7 +89,7 @@ def pagination(cl):
 
 def result_headers(cl):
     """
-    Generates the list column headers.
+    Generate the list column headers.
     """
     ordering_field_columns = cl.get_ordering_field_columns()
     for i, field_name in enumerate(cl.list_display):
@@ -104,6 +99,7 @@ def result_headers(cl):
             return_attr=True
         )
         if attr:
+            field_name = _coerce_field_name(field_name, i)
             # Potentially not sortable
 
             # if the field is the action checkbox: no sorting and special class
@@ -143,7 +139,9 @@ def result_headers(cl):
         o_list_primary = []  # URL for making this field the primary sort
         o_list_remove = []  # URL for removing this field from sort
         o_list_toggle = []  # URL for toggling order type for this field
-        make_qs_param = lambda t, n: ('-' if t == 'desc' else '') + str(n)
+
+        def make_qs_param(t, n):
+            return ('-' if t == 'desc' else '') + str(n)
 
         for j, ot in ordering_field_columns.items():
             if j == i:  # Same column
@@ -181,9 +179,21 @@ def _boolean_icon(field_val):
     return format_html('<img src="{}" alt="{}" />', icon_url, field_val)
 
 
+def _coerce_field_name(field_name, field_index):
+    """
+    Coerce a field_name (which may be a callable) to a string.
+    """
+    if callable(field_name):
+        if field_name.__name__ == '<lambda>':
+            return 'lambda' + str(field_index)
+        else:
+            return field_name.__name__
+    return field_name
+
+
 def items_for_result(cl, result, form):
     """
-    Generates the actual list of data.
+    Generate the actual list of data.
     """
 
     def link_in_col(is_first, field_name, cl):
@@ -195,9 +205,9 @@ def items_for_result(cl, result, form):
 
     first = True
     pk = cl.lookup_opts.pk.attname
-    for field_name in cl.list_display:
+    for field_index, field_name in enumerate(cl.list_display):
         empty_value_display = cl.model_admin.get_empty_value_display()
-        row_classes = ['field-%s' % field_name]
+        row_classes = ['field-%s' % _coerce_field_name(field_name, field_index)]
         try:
             f, attr, value = lookup_field(field_name, result, cl.model_admin)
         except ObjectDoesNotExist:
@@ -207,17 +217,8 @@ def items_for_result(cl, result, form):
             if f is None or f.auto_created:
                 if field_name == 'action_checkbox':
                     row_classes = ['action-checkbox']
-                allow_tags = getattr(attr, 'allow_tags', False)
                 boolean = getattr(attr, 'boolean', False)
                 result_repr = display_for_value(value, empty_value_display, boolean)
-                if allow_tags:
-                    warnings.warn(
-                        "Deprecated allow_tags attribute used on field {}. "
-                        "Use django.utils.html.format_html(), format_html_join(), "
-                        "or django.utils.safestring.mark_safe() instead.".format(field_name),
-                        RemovedInDjango20Warning
-                    )
-                    result_repr = mark_safe(result_repr)
                 if isinstance(value, (datetime.date, datetime.time)):
                     row_classes.append('nowrap')
             else:
@@ -231,7 +232,7 @@ def items_for_result(cl, result, form):
                     result_repr = display_for_field(value, f, empty_value_display)
                 if isinstance(f, (models.DateField, models.TimeField, models.ForeignKey)):
                     row_classes.append('nowrap')
-        if force_text(result_repr) == '':
+        if str(result_repr) == '':
             result_repr = mark_safe('&nbsp;')
         row_class = mark_safe(' class="%s"' % ' '.join(row_classes))
         # If list_display_links not defined, add the link tag to the first field
@@ -248,19 +249,17 @@ def items_for_result(cl, result, form):
             else:
                 url = add_preserved_filters({'preserved_filters': cl.preserved_filters, 'opts': cl.opts}, url)
                 # Convert the pk to something that can be used in Javascript.
-                # Problem cases are long ints (23L) and non-ASCII strings.
+                # Problem cases are non-ASCII strings.
                 if cl.to_field:
                     attr = str(cl.to_field)
                 else:
                     attr = pk
                 value = result.serializable_value(attr)
-                result_id = escapejs(value)
                 link_or_text = format_html(
                     '<a href="{}"{}>{}</a>',
                     url,
                     format_html(
-                        ' onclick="opener.dismissRelatedLookupPopup(window, '
-                        '&#39;{}&#39;); return false;"', result_id
+                        ' data-popup-opener="{}"', value
                     ) if cl.is_popup else '',
                     result_repr)
 
@@ -277,20 +276,21 @@ def items_for_result(cl, result, form):
                     field_name == cl.model._meta.pk.name and
                     form[cl.model._meta.pk.name].is_hidden)):
                 bf = form[field_name]
-                result_repr = mark_safe(force_text(bf.errors) + force_text(bf))
+                result_repr = mark_safe(str(bf.errors) + str(bf))
             yield format_html('<td{}>{}</td>', row_class, result_repr)
     if form and not form[cl.model._meta.pk.name].is_hidden:
-        yield format_html('<td>{}</td>', force_text(form[cl.model._meta.pk.name]))
+        yield format_html('<td>{}</td>', form[cl.model._meta.pk.name])
 
 
 class ResultList(list):
-    # Wrapper class used to return items in a list_editable
-    # changelist, annotated with the form object for error
-    # reporting purposes. Needed to maintain backwards
-    # compatibility with existing admin templates.
+    """
+    Wrapper class used to return items in a list_editable changelist, annotated
+    with the form object for error reporting purposes. Needed to maintain
+    backwards compatibility with existing admin templates.
+    """
     def __init__(self, form, *items):
         self.form = form
-        super(ResultList, self).__init__(*items)
+        super().__init__(*items)
 
 
 def results(cl):
@@ -306,13 +306,13 @@ def result_hidden_fields(cl):
     if cl.formset:
         for res, form in zip(cl.result_list, cl.formset.forms):
             if form[cl.model._meta.pk.name].is_hidden:
-                yield mark_safe(force_text(form[cl.model._meta.pk.name]))
+                yield mark_safe(form[cl.model._meta.pk.name])
 
 
 @register.inclusion_tag("admin/change_list_results.html")
 def result_list(cl):
     """
-    Displays the headers and data list together
+    Display the headers and data list together.
     """
     headers = list(result_headers(cl))
     num_sorted_fields = 0
@@ -329,12 +329,10 @@ def result_list(cl):
 @register.inclusion_tag('admin/date_hierarchy.html')
 def date_hierarchy(cl):
     """
-    Displays the date hierarchy for date drill-down functionality.
+    Display the date hierarchy for date drill-down functionality.
     """
     if cl.date_hierarchy:
         field_name = cl.date_hierarchy
-        field = cl.opts.get_field(field_name)
-        dates_or_datetimes = 'datetimes' if isinstance(field, models.DateTimeField) else 'dates'
         year_field = '%s__year' % field_name
         month_field = '%s__month' % field_name
         day_field = '%s__day' % field_name
@@ -343,7 +341,8 @@ def date_hierarchy(cl):
         month_lookup = cl.params.get(month_field)
         day_lookup = cl.params.get(day_field)
 
-        link = lambda filters: cl.get_query_string(filters, [field_generic])
+        def link(filters):
+            return cl.get_query_string(filters, [field_generic])
 
         if not (year_lookup or month_lookup or day_lookup):
             # select appropriate start level
@@ -367,7 +366,7 @@ def date_hierarchy(cl):
             }
         elif year_lookup and month_lookup:
             days = cl.queryset.filter(**{year_field: year_lookup, month_field: month_lookup})
-            days = getattr(days, dates_or_datetimes)(field_name, 'day')
+            days = getattr(days, 'dates')(field_name, 'day')
             return {
                 'show': True,
                 'back': {
@@ -381,7 +380,7 @@ def date_hierarchy(cl):
             }
         elif year_lookup:
             months = cl.queryset.filter(**{year_field: year_lookup})
-            months = getattr(months, dates_or_datetimes)(field_name, 'month')
+            months = getattr(months, 'dates')(field_name, 'month')
             return {
                 'show': True,
                 'back': {
@@ -394,7 +393,7 @@ def date_hierarchy(cl):
                 } for month in months]
             }
         else:
-            years = getattr(cl.queryset, dates_or_datetimes)(field_name, 'year')
+            years = getattr(cl.queryset, 'dates')(field_name, 'year')
             return {
                 'show': True,
                 'choices': [{
@@ -407,7 +406,7 @@ def date_hierarchy(cl):
 @register.inclusion_tag('admin/search_form.html')
 def search_form(cl):
     """
-    Displays a search form for searching the list.
+    Display a search form for searching the list.
     """
     return {
         'cl': cl,
